#include <mooutils/mooglade.h>
#include <mooutils/mooi18n.h>
#include <gtk/gtk.h>

static const char _progress_widget_xml_glade_xml[] =
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
"<glade-interface>"
"<!-- interface-requires gtk+ 2.16 -->"
"<!-- interface-naming-policy project-wide -->"
"<widget class=\"GtkWindow\" id=\"window1\">"
"<child>"
"<widget class=\"GtkAlignment\" id=\"ProgressWidget\">"
"<property name=\"xscale\">0</property>"
"<property name=\"yscale\">0</property>"
"<property name=\"top_padding\">12</property>"
"<property name=\"bottom_padding\">12</property>"
"<property name=\"left_padding\">12</property>"
"<property name=\"right_padding\">12</property>"
"<child>"
"<widget class=\"GtkTable\" id=\"table2\">"
"<property name=\"visible\">True</property>"
"<property name=\"n_columns\">2</property>"
"<property name=\"column_spacing\">6</property>"
"<child>"
"<widget class=\"GtkProgressBar\" id=\"progressbar\">"
"<property name=\"visible\">True</property>"
"<property name=\"activity_mode\">True</property>"
"<property name=\"show_text\">True</property>"
"<property name=\"text\">Doing stuff</property>"
"</widget>"
"</child>"
"<child>"
"<widget class=\"GtkButton\" id=\"cancel\">"
"<property name=\"label\">gtk-cancel</property>"
"<property name=\"visible\">True</property>"
"<property name=\"can_focus\">True</property>"
"<property name=\"receives_default\">False</property>"
"<property name=\"use_stock\">True</property>"
"<property name=\"focus_on_click\">False</property>"
"</widget>"
"<packing>"
"<property name=\"left_attach\">1</property>"
"<property name=\"right_attach\">2</property>"
"</packing>"
"</child>"
"</widget>"
"</child>"
"</widget>"
"</child>"
"</widget>"
"</glade-interface>"

;

typedef struct ProgressWidgetXml ProgressWidgetXml;

struct ProgressWidgetXml {
    MooGladeXML *xml;

    GtkAlignment *ProgressWidget;
    GtkProgressBar *progressbar;
    GtkButton *cancel;

};

static void
_progress_widget_xml_free (ProgressWidgetXml *xml)
{
    if (xml)
    {
        if (xml->xml)
            g_object_unref (xml->xml);
        g_free (xml);
    }
}

G_GNUC_UNUSED static ProgressWidgetXml *
progress_widget_xml_get (gpointer widget)
{
    return (ProgressWidgetXml*) g_object_get_data (G_OBJECT (widget), "moo-generated-glade-xml");
}

static gboolean
_progress_widget_xml_finish_build (ProgressWidgetXml *xml)
{
    g_return_val_if_fail(xml != NULL, FALSE);

    xml->ProgressWidget = (GtkAlignment*) moo_glade_xml_get_widget (xml->xml, "ProgressWidget");
    g_return_val_if_fail (xml->ProgressWidget != NULL, FALSE);
    xml->progressbar = (GtkProgressBar*) moo_glade_xml_get_widget (xml->xml, "progressbar");
    g_return_val_if_fail (xml->progressbar != NULL, FALSE);
    xml->cancel = (GtkButton*) moo_glade_xml_get_widget (xml->xml, "cancel");
    g_return_val_if_fail (xml->cancel != NULL, FALSE);

    g_object_set_data_full (G_OBJECT (xml->ProgressWidget), "moo-generated-glade-xml",
                            xml, (GDestroyNotify) _progress_widget_xml_free);
    return TRUE;
}

static gboolean
progress_widget_xml_build (ProgressWidgetXml *xml)
{
    GError *error = NULL;
    if (!moo_glade_xml_parse_memory (xml->xml, _progress_widget_xml_glade_xml, -1, "ProgressWidget", &error))
    {
        g_critical ("Could not parse glade xml: %s", error->message);
        g_error_free (error);
        return FALSE;
    }
    return _progress_widget_xml_finish_build (xml);
}

static gboolean
progress_widget_xml_fill (ProgressWidgetXml *xml, GtkWidget *root)
{
    GError *error = NULL;
    if (!moo_glade_xml_fill_widget (xml->xml, root, _progress_widget_xml_glade_xml, -1, "ProgressWidget", &error))
    {
        g_critical ("Could not parse glade xml: %s", error->message);
        g_error_free (error);
        return FALSE;
    }
    return _progress_widget_xml_finish_build (xml);
}

static ProgressWidgetXml *
progress_widget_xml_new_empty (void)
{
    ProgressWidgetXml *xml = g_new0 (ProgressWidgetXml, 1);
    xml->xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);

    return xml;
}

G_GNUC_UNUSED static ProgressWidgetXml *
progress_widget_xml_new (void)
{
    ProgressWidgetXml *xml = progress_widget_xml_new_empty ();
    if (!progress_widget_xml_build (xml))
    {
        _progress_widget_xml_free (xml);
        xml = NULL;
    }
    return xml;
}

G_GNUC_UNUSED static ProgressWidgetXml *
progress_widget_xml_new_with_root (GtkWidget *root)
{
    ProgressWidgetXml *xml = progress_widget_xml_new_empty ();
    if (!progress_widget_xml_fill (xml, root))
    {
        _progress_widget_xml_free (xml);
        xml = NULL;
    }
    return xml;
}
